/*
* luigiv4.c
*
* Created: 05.01.2015 06:15:26
*  Author: sschultewolter
*/

#include "main.h"



void time_init(void)
{
	// CTC Modus
	TCCR0A |= (1<<WGM01);
	// Prescaler 64
	TCCR0B |= 1<<CS01 | 1<<CS00;
	OCR0A = 125-1;
	// Interrupt aktivieren
	TIMSK0 |= (1<<OCIE0A);
}

uint32_t time(void) { return millis; }

ISR(TIMER0_COMPA_vect) {millis++;}

void hc595_init(void)
{
	HC595_DDR |= 1<<HC595_SS | 1<<HC595_MOSI | 1<<HC595_SCK;
	// SPI aktivieren | Master
	SPCR |= 1<<SPE | 1<<MSTR;
	// SCK = F_CPU/2
	SPSR |= 1<<SPI2X;
}

void hc595_send(uint8_t data)
{
	HC595_PORT &= ~(1<<HC595_SS);
	SPDR = data^0x0F;
	while(!(SPSR & 1<<SPIF));
	HC595_PORT |= 1<<HC595_SS;
}

void motor_init(void)
{
/*  Das geht nicht!
	MOTOR_DDR |= 1<<ALT_MOTOR_PORT;
	TOCPMSA0 |= 1<<TOCC2S0;
	TOCPMCOE |= 1<<TOCC2OE;*/

	MOTOR_DDR |= 1<<MOTOR_PORT;
	TOCPMSA0 |= 1<<TOCC1S0;
	TOCPMCOE |= 1<< TOCC1OE;

	// FastPWM
	TCCR1A = (1<<WGM10);
	// Prescaler 64
	TCCR1B = (1<<CS11) | (1<<CS10);
	MOTOR_ON;
	MOTOR_PWM = 0;
}

int main(void)
{
	time_init();
	hc595_init();
	motor_init();
	
	extern unsigned short __heap_start;
	unsigned int seed = *((unsigned int *)__heap_start);
	srand(seed);

	uint8_t random_number;
	uint32_t last_time = time();
	uint8_t led_value = 0;
	
	sei();
	
	while(1)
	{
		random_number = (uint8_t)rand();
		
		if(time() - last_time > 50)
		{
			last_time = time();
			hc595_send(led_value++);
			MOTOR_PWM = random_number;
		}
	}
}